Package: topconfects
Title: Top Confident Effect Sizes
Version: 1.28.0
Authors@R: person("Paul", "Harrison", email = "paul.harrison@monash.edu", role = c("aut", "cre"), comment=c(ORCID="0000-0002-3980-268X"))
Description: Rank results by confident effect sizes, while maintaining
        False Discovery Rate and False Coverage-statement Rate control.
        Topconfects is an alternative presentation of TREAT results
        with improved usability, eliminating p-values and instead
        providing confidence bounds. The main application is
        differential gene expression analysis, providing genes ranked
        in order of confident log2 fold change, but it can be applied
        to any collection of effect sizes with associated standard
        errors.
Depends: R (>= 3.6.0)
Imports: methods, utils, stats, assertthat, ggplot2, scales, grid,
        grDevices
Suggests: limma, edgeR, statmod, DESeq2, ashr, NBPSeq, dplyr, testthat,
        reshape2, tidyr, readr, org.At.tair.db, AnnotationDbi, knitr,
        rmarkdown, BiocStyle
License: LGPL-2.1 | file LICENSE
URL: https://github.com/pfh/topconfects
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
BugReports: https://github.com/pfh/topconfects/issues
biocViews: GeneExpression, DifferentialExpression, Transcriptomics,
        RNASeq, mRNAMicroarray, Regression, MultipleComparison
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:49:28 UTC
RemoteUrl: https://github.com/bioc/topconfects
RemoteRef: RELEASE_3_23
RemoteSha: f68dfcaf7293563e3b002f0fc699367712a96c8c
NeedsCompilation: no
Packaged: 2026-05-04 09:02:51 UTC; root
Author: Paul Harrison [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3980-268X>)
Maintainer: Paul Harrison <paul.harrison@monash.edu>
Built: R 4.6.0; ; 2026-05-04 09:20:15 UTC; unix

# Generated by roxygen2: do not edit by hand

export(confects_plot)
export(confects_plot_me)
export(confects_plot_me2)
export(deseq2_confects)
export(edger_confects)
export(limma_confects)
export(nest_confects)
export(normal_confects)
export(rank_rank_plot)
import(methods)
import(stats)
import(utils)
importFrom(assertthat,assert_that)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(grDevices,colorRampPalette)
importFrom(grid,unit)

# Topconfects 

TOP results by CONfident efFECT Size. Topconfects is intended for RNA-seq or microarray Differntial Expression analysis and similar, where we are interested in placing confidence bounds on many effect sizes--one per gene--from few samples.

Topconfects is now in Bioconductor. The github repository is for bleeding-edge development only.

* [Topconfects in Bioconductor, release version](https://bioconductor.org/packages/topconfects)
* [Topconfects in Bioconductor, development version](https://bioconductor.org/packages/devel/bioc/html/topconfects.html)

#' General purpose function to find confident effect sizes, controlling FDR
#'
#' Find sets of discoveries for a range of effect sizes, controlling the False
#' Discovery Rate (FDR) for each set.
#'
#' This is a general purpose function, which can be applied to any method of
#' calculting p-values (supplied as a function argument) for the null hypothesis
#' that the effect size is smaller than a given amount.
#'
#' @param n Number of items being tested.
#'
#' @param pfunc A \code{function(indices, effect_size)} to calculate p-values.
#'   Indices is a subset of 1:n giving the p-values to be computed. Should
#'   return a numeric vector of length \code{length(indices)}.
#'   May return NA values. 
#'   If an NA value is returned, NA must be returned for all effect sizes. 
#'   NA values are not counted toward the total number of tests performed.
#'
#' @param fdr False Discovery Rate to control for.
#'
#' @param step Granularity of effect sizes to test.
#'
#' @param full If TRUE, also include FDR-adjusted p-value that effect size is
#'   non-zero. Note that this is against the spirit of the topconfects approach.
#'
#' @return
#'
#' A "Topconfects" object, containing a table of results and various associated
#' information.
#'
#' The most important part of this object is the $table element, a data frame
#' with the following columns:
#'
#' \itemize{ \item \code{rank} - Ranking by \code{confect} and for equal
#' \code{confect} by p-value at that effect size. \item \code{index} - Number of
#' the test, between 1 and n. \item \code{confect} - CONfident efFECT size. }
#'
#' The usage is as follows: To find a set of tests which have effect size
#' greater than x with the specified FDR, take the rows with \code{abs(confect)
#' >= x}. Once the set is selected, the confect values provide confidence bounds
#' on the effect size with False Coverage-statement Rate (FCR) at the same level
#' as the FDR.
#'
#' One may essentially take the top however many rows of the data frame and
#' these will be the best set of results of that size to dependably have an
#' effect size that is as large as possible. However if some genes have the same
#' \code{abs(confect)}, all or none should be selected.
#'
#' Some rows in the output may be given the same \code{confect}, even if
#' \code{step} is made small. This is an expected behaviour of the algorithm.
#' (This is similar to FDR adjustment of p-values sometimes resulting in a run
#' of the same adjusted p-value, even if all the input p-values are distinct.)
#'
#' Some wrappers around this function may add a sign to the \code{confect}
#' column, if it makes sense to do so. They will also generally add an
#' \code{effect} column, containing an estimate of the effect size that aims to
#' be unbiassed rather than a conservative lower bound.
#'
#' @examples
#'
#' # Find largest positive z-scores in a collection,
#' # and place confidence bounds on them that maintain FDR 0.05.
#' z <- c(1,2,3,4,5)
#' pfunc <- function(i, effect_size) {
#'     pnorm(z[i], mean=effect_size, lower.tail=FALSE)
#' }
#' nest_confects(length(z), pfunc, fdr=0.05)
#'
#' @export
nest_confects <- function(n, pfunc, fdr=0.05, step=0.001, full=FALSE) {
    indices <- seq_len(n)
    mags <- rep(NA, n)

    # Calculate all p-values with effect size 0.
    # This lets us work out if any are missing.
    pfunc_all_0 <- pfunc(indices, 0.0)
    good <- !is.na(pfunc_all_0)
    n_good <- sum(good)

    indices <- c(indices[good], indices[!good])
    n_active <- n_good

    steps <- 0
    while(n_active > 0) {
        mag <- steps * step
        seq_active <- seq_len(n_active)

        # Re-use original pfunc call first time around
        if (mag == 0)
            p <- pfunc_all_0[indices[seq_active]]
        else
            p <- pfunc(indices[seq_active], mag)
        
        ordering <- order(p)
        p <- p[ordering]
        indices[seq_active] <- indices[ordering]
        mags[seq_active] <- mags[ordering]

        while(n_active > 0 && p[n_active] > fdr*n_active/n_good)
            n_active <- n_active - 1

        mags[seq_len(n_active)] <- mag
        steps <- steps + 1
    }

    table <- data.frame(rank=seq_len(n), index=indices, confect=mags)
    if (full) {
        table$fdr_zero <-
            p.adjust(pfunc_all_0, n=n_good, method="BH")[ table$index ]
    }

    new("Topconfects", list(
        table=table,
        effect_desc = "effect size",
        fdr=fdr, step=step, pfunc=pfunc))
}

#
# Ensure a numeric vector has a certain length.
# Repeat input if input length is 1.
# Strip any names if present.
#
broadcast <- function(vec, n) {
    assert_that(is.numeric(vec))
    names(vec) <- NULL
    if (length(vec) == 1)
        vec <- rep(vec, n)
    assert_that(length(vec) == n)
    vec
}

#' Confident log2 fold changes based on a limma fit object
#'
#' For all possible absolute log2 fold changes (LFC), which genes have at least
#' this fold change at a specified False Discovery Rate (FDR)?
#'
#' Results are presented in a table such that for any given LFC, if the reader
#' chooses the genes with \code{abs(confect)} less than this they are assured
#' that this set of genes has at least this LFC (with the specified FDR). Once
#' this set of genes is selected, the confect values provide confidence bounds
#' with False Coverage-statement Rate at the same level as the FDR.
#'
#' \code{fit} should be produced using \code{lmFit}. It is not necessary to use
#' \code{eBayes}, this function calls \code{eBayes} itself.
#'
#' To test contrasts, this function can also be used with the result of
#' \code{contrasts.fit}, but limma's handling of weights may be approximate (for
#' example if \code{voom} has been used). For exact results for a contrast, use
#' \code{contrastToCoef} to adjust the design matrix given to \code{lmFit}.
#'
#' @param fit A limma MArrayLM object.
#'
#' @param coef Number or name of coefficient or contrast to test.
#'
#' @param fdr False Discovery Rate to control for.
#'
#' @param step Granularity of log2 fold changes to test.
#'
#' @param trend Should \code{eBayes(trend=TRUE)} be used?
#'
#' @param full Include some further statistics used to calculate confects in the
#'   output, and also include FDR-adjusted p-values that effect size is non-zero
#'   (note that this is against the spirit of the topconfects approach).
#'
#' @return
#'
#' See \code{\link{nest_confects}} for details of how to interpret the result.
#'
#' @examples
#'
#' #Prepare a data set
#' library(NBPSeq)
#' library(edgeR)
#' library(limma)
#' data(arab)
#' dgelist <- DGEList(arab)
#' dgelist <- calcNormFactors(dgelist)
#' cpms <- cpm(dgelist, log=TRUE)
#' # Retain genes with more than a geometric mean of 2 RPM
#' # (about 5 reads per sample)
#' cpms <- cpms[rowMeans(cpms) >= 1,]
#'
#' # Fit linear model for each gene
#' treatment <- c(FALSE,FALSE,FALSE,TRUE,TRUE,TRUE)
#' batch <- factor(c(1,2,3,1,2,3))
#' design <- model.matrix(~ treatment + batch)
#' fit <- lmFit(cpms, design)
#'
#' # Calculate top confects
#' # As voom has not been used, it is necessary to use trend=TRUE
#' limma_confects(fit, "treatmentTRUE", trend=TRUE)
#'
#' @export
limma_confects <- function(
        fit, coef=NULL, fdr=0.05, step=0.001, trend=FALSE, full=FALSE) {
    assert_that(requireNamespace("limma", quietly=TRUE), msg=
        "limma_confects requires installing the Bioconductor package 'limma'")
    assert_that(is(fit, "MArrayLM"), msg="fit must be an MArrayLM object")

    if (is.null(coef) && ncol(fit$coefficients) == 1)
        coef <- 1

    assert_that(length(coef)==1, msg="Need to specify one coefficient to test")

    n <- nrow(fit)

    fit <- limma::eBayes(fit, trend=trend)

    effect <- fit$coefficients[,coef]
    sd <- fit$stdev.unscaled[,coef] * sqrt(fit$s2.post)
    df <- fit$df.total
    confects <- normal_confects(effect, sd, df, fdr=fdr, step=step, full=full)

    confects$table$AveExpr <- fit$Amean[confects$table$index]
    if (!is.null(rownames(fit)))
        confects$table$name <- rownames(fit)[confects$table$index]
    else
        confects$table$name <- as.character(confects$table$index)
    confects$effect_desc <- "log2 fold change"

    confects$limma_fit <- fit

    if (!is.null(fit$genes)) {
        confects$table <- cbind(
            confects$table,
            fit$genes[confects$table$index,,drop=FALSE])
    }

    confects
}


#
# Version of limma_confects using the limma treat function.
#
limma_confects_limma <- function(
        fit, coef=NULL, fdr=0.05, step=0.001, trend=FALSE) {
    assert_that(requireNamespace("limma", quietly=TRUE), msg=
    "limma_confects_limma requires installing the Bioconductor package 'limma'")
    assert_that(is(fit, "MArrayLM"), msg="fit must be an MArrayLM object")

    if (is.null(coef) && ncol(fit$coefficients) == 1)
        coef <- 1

    assert_that(length(coef)==1, msg="Need to specify one coefficient to test")

    n <- nrow(fit)

    pfunc <- function(i, mag) {
        tested <- limma::treat(fit, lfc=mag, trend=trend)
        top_treats <- limma::topTreat(tested, coef=coef, sort.by="none",n=n)
        top_treats$P.Value[i]
    }

    confects <- nest_confects(n, pfunc, fdr=fdr, step=step)
    logFC <- fit$coefficients[confects$table$index, coef]
    confects$table$confect <- sign(logFC) * confects$table$confect
    confects$table$effect <- logFC

    confects$table$AveExpr <- fit$Amean[confects$table$index]
    if (!is.null(rownames(fit)))
        confects$table$name <- rownames(fit)[confects$table$index]
    else
        confects$table$name <- as.character(confects$table$index)
    confects$effect_desc <- "log2 fold change"

    confects$limma_fit <- fit

    if (!is.null(fit$genes)) {
        confects$table <- cbind(
            confects$table,
            fit$genes[confects$table$index,,drop=FALSE])
    }

    confects
}

context("topconfects TREAT implementation should exactly match limma's")

library(limma)
library(topconfects)

set.seed(1234)
n <- 100
folds <- seq(-4,4,length.out=n)
row_means <- runif(n, min=0, max=5)
means <- outer(folds, c(-0.5,-0.5,0.5,0.5)) + row_means
sds <- rep(runif(n, min=0,max=2), 4)
mat <- rnorm(length(means), mean=means, sd=sds)
dim(mat) <- dim(means) 

design <- cbind(c(1,1,0,0), c(0,0,1,1))

fit <- lmFit(mat, design)
cfit <- contrasts.fit(fit, contrasts=c(-1,1))

# Using our version of limma's treat
confects_standard <- limma_confects(cfit, coef=1)
# Using limma's treat function
confects_limma <- topconfects:::limma_confects_limma(cfit, coef=1)

# May as well also test FCR guarantee is met
achieved_fcrs <- cumsum(folds[confects_standard$table$index]/confects_standard$table$confect <= 1) / (1:n)

capture.output(file="test_limma_identical.out", {
    cat("\nUsing topconfects implementation\n")
    print(confects_standard)
    cat("\nUsing limma\n")
    print(confects_limma)
    cat("\nAchieved FCR\n")
    print(achieved_fcrs)
})

test_that("tables identical", {
    expect_equal(confects_standard$table, confects_limma$table)
})

test_that("p values identical", {
    for(lfc in c(0.5, 1.0, 2.0)) {
        p <- confects_standard$pfunc(1:n, lfc)
        p_limma <- confects_limma$pfunc(1:n, lfc)
        expect_equal(p, p_limma)
    }
})

test_that("FCR of 5% for all heads of table", {
    expect(all(achieved_fcrs <= 0.05, na.rm=TRUE), "FCR not controlled")
})

#' Confident log2 fold change based on the edgeR Quasi-Likelihood method
#'
#' For all possible absolute log2 fold changes (LFC), which genes have at least
#' this fold change at a specified False Discovery Rate?
#'
#' Results are presented in a table such that for any given LFC, if the reader
#' chooses the genes with abs(confect) less than this they are assured that this
#' set of genes has at least this LFC (with the specified FDR). The confect
#' column may also be viewed as a confidence bound on the LFC of each gene, with
#' a dynamic correction for multiple testing.
#'
#' @param fit An edgeR DGEGLM object produced using \code{glmQLFit}.
#'
#' @param coef Coefficient to test, as per \code{glmTreat}. Use either coef or
#'   contrast or effect.
#'
#' @param contrast Contrast to test, as per \code{glmTreat}. Use either coef or
#'   contrast or effect.
#'
#' @param fdr False Discovery Rate to control for.
#'
#' @param step Granularity of log2 fold changes to test.
#'
#' @param null "null" parameter passed through to edger::glmTreat (if coef or
#'   contrast given). Choices are "worst.case" or "interval". Note that the
#'   default here is "worst.case", to be consistent with other functions in
#'   topconfects. This differs from the default for glmTreat.
#'
#' @return
#'
#' See \code{\link{nest_confects}} for details of how to interpret the result.
#'
#' @examples
#'
#' # Generate some random data
#' n <- 100
#' folds <- seq(-4,4,length.out=n)
#' row_means <- runif(n, min=0, max=5)
#' lib_scale <- c(1,2,3,4)
#' means <- 2^(outer(folds, c(-0.5,-0.5,0.5,0.5))) *
#'     row_means * rep(lib_scale,each=n)
#' counts <- rnbinom(length(means), mu=means, size=1/0.1)
#' dim(counts) <- dim(means)
#'
#' design <- cbind(c(1,1,0,0), c(0,0,1,1))
#'
#' # Fit data using edgeR quasi-likelihood
#' library(edgeR)
#' y <- DGEList(counts)
#' y <- calcNormFactors(y)
#' y <- estimateDisp(y, design)
#' fit <- glmQLFit(y, design)
#'
#' # Find top confident effect sizes
#' edger_confects(fit, contrast=c(-1,1))
#'
#' @export
edger_confects <- function(
        fit, coef=NULL, contrast=NULL,
        fdr=0.05, step=0.01, null=c("worst.case","interval")) {
    assert_that(requireNamespace("edgeR", quietly=TRUE), msg=
        "edger_confects requires installing the Bioconductor package 'edgeR'")
    assert_that(is(fit, "DGEGLM"))
    assert_that((!is.null(coef)) + (!is.null(contrast)) == 1)

    null <- match.arg(null)

    n <- nrow(fit)

    tested <- edgeR::glmQLFTest(fit,coef=coef,contrast=contrast)
    top_tags <- edgeR::topTags(tested, n=n,sort.by="none")

    pfunc <- function(i, mag) {
        if (mag == 0.0)
            top_treats <- top_tags
        else {
            treat_tested <- edgeR::glmTreat(
                fit, coef=coef, contrast=contrast, lfc=mag, null=null)
            top_treats <- edgeR::topTags(treat_tested, n=n, sort.by="none")
        }

        top_treats$table$PValue[i]
    }

    confects <- nest_confects(n, pfunc, fdr=fdr, step=step)
    confects$effect_desc <- "log2 fold change"
    logFC <- top_tags$table$logFC[confects$table$index]
    confects$table$confect <- sign(logFC) * confects$table$confect
    confects$table$effect <- logFC
    confects$table$logCPM <- top_tags$table$logCPM[confects$table$index]
    confects$table$name <- rownames(top_tags$table)[confects$table$index]

    confects$edger_fit <- fit

    if (!is.null(fit$genes)) {
        confects$table <- cbind(
            confects$table,
            fit$genes[confects$table$index,,drop=FALSE])
    }

    confects
}

context("Test edgeR-based routines")

# test_edger.out can be examined

library(edgeR)
library(topconfects)

set.seed(1234)

n <- 100
folds <- seq(-4,4,length.out=n)
row_means <- runif(n, min=0, max=5)
lib_scale <- c(1,2,3,4)
means <- 2^(outer(folds, c(-0.5,-0.5,0.5,0.5))) * row_means * rep(lib_scale,each=n)
counts <- rnbinom(length(means), mu=means, size=1/0.1)
dim(counts) <- dim(means) 

design <- cbind(c(1,1,0,0), c(0,0,1,1))

y <- DGEList(counts)
y <- calcNormFactors(y)
y <- estimateDisp(y, design)

fit <- glmQLFit(y, design)

set.seed(12345)
confects <- edger_confects(fit, contrast=c(-1,1))

achieved_fcrs <- cumsum(folds[confects$table$index]/confects$table$confect <= 1) / (1:n)

capture.output(file="test_edger.out", {
    cat("\ncounts\n")
    print(counts)
    cat("\ncolSums(counts)\n")
    print(colSums(counts))
    cat("\ndf.prior\n")
    print(fit$df.prior)
    cat("\nconfects\n")
    print(confects)
    cat("\nconfects$table\n")
    print(confects$table)
    cat("\nAchieved FCRs\n")
    print(achieved_fcrs)
})

test_that("FCR of 5% for all heads of table", {
    expect(all(achieved_fcrs <= 0.05, na.rm=TRUE), "FCR not controlled")
})

#' @import methods
#' @import utils
#' @import stats
#' @importFrom ggplot2
#'   aes_string geom_hline geom_point geom_segment geom_text
#'   geom_vline ggplot labs scale_x_continuous scale_y_continuous scale_size
#'   scale_color_discrete scale_color_manual guides guide_legend theme theme_bw
#'   coord_cartesian
#' @importFrom grid
#'   unit
#' @importFrom grDevices
#'   colorRampPalette
#' @importFrom assertthat
#'   assert_that
#'
NULL

params <-
list(cache = FALSE)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(fig.width=7, fig.height=6)

## ----load, message=FALSE, warning=FALSE---------------------------------------
library(topconfects)

library(NBPSeq)
library(edgeR)
library(limma)

library(dplyr)
library(ggplot2)

data(arab)

# Retrieve symbol for each gene
info <- 
    AnnotationDbi::select(
        org.At.tair.db::org.At.tair.db, 
        rownames(arab), "SYMBOL") %>%
    group_by(TAIR) %>% 
    summarize(
        SYMBOL=paste(unique(na.omit(SYMBOL)),collapse="/"))
arab_info <- 
    info[match(rownames(arab),info$TAIR),] %>% 
    select(-TAIR) %>%
    as.data.frame
rownames(arab_info) <- rownames(arab)

# Extract experimental design from sample names
Treat <- factor(substring(colnames(arab),1,4)) %>% relevel(ref="mock")
Time <- factor(substring(colnames(arab),5,5))

y <- DGEList(arab, genes=as.data.frame(arab_info))

# Keep genes with at least 3 samples having an RPM of more than 2
keep <- rowSums(cpm(y)>2) >= 3
y <- y[keep,,keep.lib.sizes=FALSE]
y <- calcNormFactors(y)

## ----limma--------------------------------------------------------------------
design <- model.matrix(~Time+Treat)
design[,]

fit <-
    voom(y, design) %>%
    lmFit(design)

## ----limma_confects-----------------------------------------------------------
confects <- limma_confects(fit, coef="Treathrcc", fdr=0.05)

confects

## ----fig.height=7-------------------------------------------------------------
confects_plot(confects)

## -----------------------------------------------------------------------------
confects_plot_me2(confects, breaks=c(0,1,2))

## -----------------------------------------------------------------------------
confects_plot_me(confects) + labs(title="This plot is not recommended")

## ----fig.height=7-------------------------------------------------------------
fit_eb <- eBayes(fit)
top <- topTable(fit_eb, coef="Treathrcc", n=Inf)

rank_rank_plot(confects$table$name, rownames(top), "limma_confects", "topTable")

## -----------------------------------------------------------------------------
plotMD(fit, legend="bottomleft", status=paste0(
    ifelse(rownames(fit) %in% rownames(top)[1:40], "topTable ",""),
    ifelse(rownames(fit) %in% confects$table$name[1:40], "confects ","")))

## ----edger--------------------------------------------------------------------
y <- estimateDisp(y, design, robust=TRUE)
efit <- glmQLFit(y, design, robust=TRUE)

## ----edger_confects-----------------------------------------------------------
econfects <- edger_confects(efit, coef="Treathrcc", fdr=0.05, step=0.05)

econfects

## ----fig.height=7-------------------------------------------------------------
confects_plot(econfects)
confects_plot_me2(econfects, breaks=c(0,1,2))

## -----------------------------------------------------------------------------
etop <-
    glmQLFTest(efit, coef="Treathrcc") %>%
    topTags(n=Inf)

plotMD(efit, legend="bottomleft", status=paste0(
    ifelse(rownames(efit) %in% econfects$table$name[1:40], "confects ", ""),
    ifelse(rownames(efit) %in% rownames(etop)[1:40], "topTags ","")))

## ----message=F, warning=F-----------------------------------------------------
library(DESeq2)

dds <- DESeqDataSetFromMatrix(
    countData = arab,
    colData = data.frame(Time, Treat),
    rowData = arab_info,
    design = ~Time+Treat)

dds <- DESeq(dds)

## -----------------------------------------------------------------------------
dconfects <- deseq2_confects(dds, name="Treat_hrcc_vs_mock", step=0.05)

## -----------------------------------------------------------------------------
shrunk <- lfcShrink(dds, coef="Treat_hrcc_vs_mock", type="ashr")
dconfects$table$shrunk <- shrunk$log2FoldChange[dconfects$table$index]

dconfects

## -----------------------------------------------------------------------------
table(dconfects$table$filtered)
tail(dconfects$table)

## ----fig.height=7-------------------------------------------------------------
confects_plot(dconfects) + 
    geom_point(aes(x=shrunk, size=baseMean, color="lfcShrink"), alpha=0.75)

## -----------------------------------------------------------------------------
filter(dconfects$table, !filtered) %>%
ggplot(aes(
        x=ifelse(is.na(confect),0,confect), y=shrunk, color=!is.na(confect))) +
    geom_point() + geom_abline() + coord_fixed() + theme_bw() +
    labs(color="Significantly\nnon-zero at\nFDR 0.05", 
        x="confect", y="lfcShrink using ashr")

## ----fig.height=7-------------------------------------------------------------
rank_rank_plot(confects$table$name, econfects$table$name, 
    "limma confects", "edgeR confects")
rank_rank_plot(confects$table$name, dconfects$table$name, 
    "limma confects", "DESeq2 confects")
rank_rank_plot(econfects$table$name, dconfects$table$name, 
    "edgeR confects", "DESeq2 confects")

## -----------------------------------------------------------------------------
sessionInfo()

#
# Topconfects result class definition
#

setClass("Topconfects", representation("list"))

setMethod("show", "Topconfects", function(object) {
    cat("$table\n")
    sub <- head(object$table, 10)
    rownames(sub) <- sub$rank
    sub$rank <- NULL
    sub$index <- NULL
    print.data.frame(sub, digits=4, right=FALSE)
    if (nrow(object$table) > 10) cat("...\n")
    cat(confects_description(object))
})

first_match <- function(options, avail, default=NULL) {
    good <- options[options %in% avail]
    if (length(good) == 0) return(default)
    good[1]
}

#
# Describe some key numbers from a result (used by show()).
#
confects_description <- function(confects) {
    result <- paste0(
        sum(!is.na(confects$table$confect)),
        " of ", nrow(confects$table), " non-zero ", confects$effect_desc,
        " at FDR ", confects$fdr, "\n")

    if (!is.null(confects$df_prior)) {
        result <- paste0(result,
            "Prior df ", sprintf("%.1f", confects$df_prior), "\n")
    }

    if (!is.null(confects$edger_fit) &&
            length(confects$edger_fit$df.prior) == 1) {
        result <- paste0(result,
            "Prior df ", sprintf("%.1f", confects$edger_fit$df.prior), "\n")
    }

    if (!is.null(confects$limma_fit) &&
            length(confects$limma_fit$df.prior) == 1) {
        result <- paste0(result,
            "Prior df ", sprintf("%.1f", confects$limma_fit$df.prior), "\n")
    }

    if (!is.null(confects$edger_fit$dispersion)) {
        result <- paste0(result,
            sprintf("Dispersion %#.2g to %#.2g\n",
                min(confects$edger_fit$dispersion),
                max(confects$edger_fit$dispersion)),
            sprintf("Biological CV %.1f%% to %.1f%%\n",
                100*sqrt(min(confects$edger_fit$dispersion)),
                100*sqrt(max(confects$edger_fit$dispersion))))
    }

    result
}


# Helper function for plots
get_limits <- function(tab, limits, expansion=1) {
    if (is.null(limits))
        limits <- c(NA,NA)
    
    min_effect <- min(0, tab$effect, na.rm=TRUE)
    max_effect <- max(0, tab$effect, na.rm=TRUE)
    
    if (min_effect == max_effect) {
        min_effect <- -1
        max_effect <- 1
    }
    
    if (is.na(limits[1]) & is.na(limits[2])) {
        max_abs_effect <- max(-min_effect,max_effect)
        limits <- c(-max_abs_effect*expansion, max_abs_effect*expansion)
    } else if (is.na(limits[1])) {
        limits[1] <- min_effect * expansion
    } else if (is.na(limits[2])) {
        limits[2] <- max_effect * expansion
    }
    
    assert_that(is.numeric(limits), length(limits) == 2)
    
    limits
}

# Red colors
get_reds <- function(n_breaks) {
    reds <- c(
        "#fcae91",
        "#fb6a4a",
        "#de2d26",
        "#a50f15")
    tail(colorRampPalette(reds)(max(2,n_breaks)),n_breaks)
}

# Blue colors
get_blues <- function(n_breaks) {
    blues <- c(
        "#6baed6",
        "#3182bd",
        "#08519c",
        "#08306b")
    tail(colorRampPalette(blues)(max(2,n_breaks)),n_breaks)
}


#' Top confident effect sizes plot
#'
#' Create a ggplot2 object showing the confect, effect, and average expression
#' level of top features in a Topconfects object.
#'
#' For each gene, the estimated effect is shown as a dot. The confidence bound
#' is shown as a line to positive or negative infinity, showing the set of
#' non-rejected effect sizes for the feature.
#'
#' @param confects A "Topconfects" class object, as returned from
#'   limma_confects, edger_confects, etc.
#'
#' @param n Number if items to show.
#'
#' @param limits c(lower, upper) limits on x-axis.
#'
#' @return
#'
#' A ggplot2 object. Working non-interactively, you must print() this for it to
#' be displayed.
#'
#' @examples
#'
#' # Generate some random effect sizes with random accuracies
#' n <- 100
#' effect <- rnorm(n, sd=2)
#' se <- rchisq(n, df=3)^-0.5
#'
#' # Find top confident effect sizes
#' confects <- normal_confects(effect, se)
#'
#' # Plot top confident effect sizes
#' confects_plot(confects, n=30)
#'
#' @export
confects_plot <- function(confects, n=50, limits=NULL) {
    tab <- head(confects$table, n)

    mag_col <- confects$magnitude_column
    if (is.null(mag_col)) {
        mag_col <- first_match(
            c("logCPM", "AveExpr", "row_mean", "baseMean"), names(tab))
    }

    mag_desc <- confects$magnitude_desc
    if (is.null(mag_desc)) {
        mag_desc <- mag_col
    }
    
    name_col <- first_match(
        c("name", "index"), names(tab))
    
    if (identical(mag_col,"baseMean"))
        mag_scale <- "log10"
    else
        mag_scale <- "identity"
    
    if (is.null(limits))
        limits <- confects$limits
    limits <- get_limits(tab, limits, 1.05)
    
    tab$confect_from <- limits[1]
    tab$confect_to <- limits[2]
    positive <- !is.na(tab$confect) & tab$effect > 0
    tab$confect_from[positive] <- tab$confect[positive]
    negative <- !is.na(tab$confect) & tab$effect < 0
    tab$confect_to[negative] <- tab$confect[negative]

    tab$name <- factor(tab[[name_col]],rev(tab[[name_col]]))

    p <- ggplot(tab, aes_string(y="name", x="effect")) +
        geom_vline(xintercept=0) +
        geom_segment(aes_string(
            yend="name", x="confect_from", xend="confect_to")) +
        geom_point(aes_string(size=mag_col)) +
        scale_x_continuous(expand=c(0,0), limits=limits, oob=function(a,b) a) +
        labs(x = confects$effect_desc, y="", size=mag_desc) +
        theme_bw()

    if (identical(mag_col,"baseMean"))
        p <- p + scale_size(trans="log10")

    p
}

#' Mean-expression vs effect size plot (deprecated)
#'
#' Note: I now recommend using \code{plot_confects_me2} instead of this plot. 
#' Like plotMD in limma, plots effect size against mean expression level.
#' However shows "confect" on the y axis rather than "effect" ("effect" is shown
#' underneath in grey). This may be useful for assessing whether effects are
#' only being detected in highly expressed genes.
#'
#' @param confects A "Topconfects" class object, as returned from
#'   \code{limma_confects}, \code{edger_confects}, or \code{deseq2_confects}.
#'
#' @return
#'
#' The two types of points in this plot make it quite confusing to explain.
#' \code{plot_confects_me2} is recommended instead.
#'
#' A ggplot2 object. Working non-interactively, you must print() this for it to
#' be displayed.
#'
#' @examples
#'
#' library(NBPSeq)
#' library(edgeR)
#' library(limma)
#'
#' data(arab)
#'
#' # Extract experimental design from sample names
#' treat <- factor(substring(colnames(arab),1,4), levels=c("mock","hrcc"))
#' time <- factor(substring(colnames(arab),5,5))
#'
#' # Keep genes with at least 3 samples having an RPM of more than 2
#' y <- DGEList(arab)
#' keep <- rowSums(cpm(y)>2) >= 3
#' y <- y[keep,,keep.lib.sizes=FALSE]
#' y <- calcNormFactors(y)
#'
#' # Find top confident fold changes by topconfects-limma-voom method
#' design <- model.matrix(~time+treat)
#' voomed <- voom(y, design)
#' fit <- lmFit(voomed, design)
#' confects <- limma_confects(fit, "treathrcc")
#'
#' # Plot confident effect size against mean expression
#' # (estimated effect size also shown as grey dots)
#' confects_plot_me(confects)
#'
#' @export
confects_plot_me <- function(confects) {
    tab <- confects$table

    mag_col <- confects$magnitude_column
    if (is.null(mag_col)) {
        mag_col <- first_match(
            c("logCPM", "AveExpr", "row_mean", "baseMean"), names(tab))
    }

    assert_that(!is.null(mag_col), msg="No mean expression column available.")

    mag_desc <- confects$magnitude_desc
    if (is.null(mag_desc)) {
        mag_desc <- mag_col
    }
    
    non_na_tab <- tab[!is.na(tab$confect),]
    non_na_tab$color <- ifelse(non_na_tab$effect >= 0, "#cc0000", "#0000cc")
    
    p <- ggplot(tab, aes_string(x=mag_col)) +
        geom_point(aes_string(y="effect"), color="#cccccc") +
        geom_hline(yintercept=0) +
        geom_point(data=non_na_tab, mapping=aes_string(y="confect"), color=non_na_tab$color) +
        labs(x=mag_desc, y=confects$effect_desc) +
        theme_bw()

    if (identical(mag_col,"baseMean"))
        p <- p + scale_x_continuous(trans="log10")

    p
}



#' Mean-expression vs effect size plot (version 2)
#'
#' Like plotMD in limma, plots effect size against mean expression level.
#' Confect values are indicated using color.
#' This may be useful for assessing whether effects are
#' only being detected in highly expressed genes.
#'
#' @param confects A "Topconfects" class object, as returned from
#'   \code{limma_confects}, \code{edger_confects}, or \code{deseq2_confects}.
#'
#' @param breaks A vector of non-negative confect thresholds to color points with.
#'   Chooses a sensible set of breaks if none are given.
#'
#' @param signed_colors Should positive and negative confects be colored 
#'   distinctly, red and blue.
#'
#' @return
#'
#' A ggplot2 object. Working non-interactively, you must print() this for it to
#' be displayed.
#'
#' @examples
#'
#' library(NBPSeq)
#' library(edgeR)
#' library(limma)
#'
#' data(arab)
#'
#' # Extract experimental design from sample names
#' treat <- factor(substring(colnames(arab),1,4), levels=c("mock","hrcc"))
#' time <- factor(substring(colnames(arab),5,5))
#'
#' # Keep genes with at least 3 samples having an RPM of more than 2
#' y <- DGEList(arab)
#' keep <- rowSums(cpm(y)>2) >= 3
#' y <- y[keep,,keep.lib.sizes=FALSE]
#' y <- calcNormFactors(y)
#'
#' # Find top confident fold changes by topconfects-limma-voom method
#' design <- model.matrix(~time+treat)
#' voomed <- voom(y, design)
#' fit <- lmFit(voomed, design)
#' confects <- limma_confects(fit, "treathrcc")
#'
#' # Plot confident effect size against mean expression
#' # (estimated effect size also shown as grey dots)
#' confects_plot_me2(confects)
#'
#' @export
confects_plot_me2 <- function(confects, breaks=NULL, signed_colors=TRUE) {
    tab <- confects$table
    
    mag_col <- confects$magnitude_column
    if (is.null(mag_col)) {
        mag_col <- first_match(
            c("logCPM", "AveExpr", "row_mean", "baseMean"), names(tab))
    }
    
    assert_that(!is.null(mag_col), msg="No mean expression column available.")
    
    mag_desc <- confects$magnitude_desc
    if (is.null(mag_desc)) {
        mag_desc <- mag_col
    }
    
    y_limits <- get_limits(tab, confects$limits, 1)
    
    is_unsigned <- y_limits[1] >= 0
    
    # Provide sensible breaks if not given
    if (length(breaks) == 0) {
        breaks <- 0.0
        max_confect <- max(0.0, abs(tab$confect), na.rm=TRUE)
        big_step <- 0.1
        repeat {
            step <- big_step
            if (step*4 >= max_confect) break
            step <- big_step*2
            if (step*4 >= max_confect) break
            step <- big_step*5
            if (step*4 >= max_confect) break
            big_step <- big_step*10
        }
        breaks <- seq(0, max_confect, by=step)
    }
    
    assert_that(all(breaks >= 0), msg="Breaks should be non-negative.")
    
    n_breaks <- length(breaks)
    breaks <- sort(unique(breaks))
    if (signed_colors) {
        pos_label <- paste0("> ", breaks)
        neg_label <- paste0("< ", -breaks)
        tab$color <- rep(NA_character_, nrow(tab))
        for(i in seq_len(n_breaks)) {
            tab$color[ !is.na(tab$confect) & tab$effect > 0 & abs(tab$confect) >= breaks[i] ] <- pos_label[i]
            if (!is_unsigned) 
                tab$color[ !is.na(tab$confect) & tab$effect < 0 & abs(tab$confect) >= breaks[i] ] <- neg_label[i]
        }
        tab$color <- factor(tab$color, c(rev(pos_label), if (!is_unsigned) neg_label))
        
        colors <- c(
            rev(get_reds(n_breaks)),
            if (!is_unsigned) get_blues(n_breaks))
        
        color_label <- paste0(confects$effect_desc,"\nconfidently")
    } else {
        tab$color <- rep(NA_character_, nrow(tab))
        label <- paste0("> ", breaks)
        for(i in seq_len(n_breaks)) {
            tab$color[ !is.na(tab$confect) & abs(tab$confect) >= breaks[i] ] <- label[i]
        }
        tab$color <- factor(tab$color, rev(label))
        colors <- scales::pal_hue(h=c(180,380),direction=-1)(n_breaks)
        if (is_unsigned)
            color_label <- paste0(confects$effect_desc,"\nconfidently")
        else
            color_label <- paste0("|",confects$effect_desc,"|\nconfidently")
    }
    
    # Reverse order
    tab <- tab[rev(seq_len(nrow(tab))),,drop = FALSE]
    
    p <- ggplot(tab, aes_string(x=mag_col, y="effect", color="color")) +
        geom_point(show.legend=TRUE) +
        scale_color_manual(
            breaks=levels(tab$color),
            values=colors,
            na.value="#dcdcdc", 
            drop=FALSE) +
        guides(color=guide_legend(override.aes=list(size=4))) +
        labs(
            x=mag_desc, 
            y=confects$effect_desc,
            color=color_label) +
        geom_hline(yintercept=0) +
        coord_cartesian(ylim=y_limits) +
        theme_bw() +
        theme(legend.key.size = unit(1, 'lines'))
    
    if (identical(mag_col,"baseMean"))
        p <- p + scale_x_continuous(trans="log10")
    
    p
}



#' A plot to compare two rankings
#'
#' This is useful, for example, when comparing different methods of ranking
#' potentially interesting differentially expressed genes.
#'
#' @param vec1 A vector of names.
#'
#' @param vec2 Another vector of names.
#'
#' @param label1 A label to go along with vec1.
#'
#' @param label2 A label to go along with vec2.
#'
#' @param n Show at most the first n names in vec1 and vec2.
#'
#' @return
#'
#' A ggplot2 object. Working non-interactively, you must print() this for it to
#' be displayed.
#'
#' @examples
#'
#' a <- sample(letters)
#' b <- sample(letters)
#' rank_rank_plot(a,b, n=20)
#'
#' @export
rank_rank_plot <- function(
        vec1, vec2, label1="First ranking", label2="Second ranking", n=40) {
    vec1 <- as.character( head(vec1, n) )
    vec2 <- as.character( head(vec2, n) )

    df1 <- data.frame(
        rank1=seq_len(length(vec1)), name=vec1, stringsAsFactors=FALSE)
    df2 <- data.frame(
        rank2=seq_len(length(vec2)), name=vec2, stringsAsFactors=FALSE)
    link <- merge(df1, df2, by="name")
    p <- ggplot(link) +
        geom_segment(aes_string(x="1", xend="2", y="rank1", yend="rank2")) +
        geom_point(aes_string(x="1", y="rank1")) +
        geom_point(aes_string(x="2", y="rank2")) +
        geom_text(data=df1, aes_string(x="0.9",y="rank1",label="name"),
            hjust=1,vjust=0.5) +
        geom_text(data=df2, aes_string(x="2.1",y="rank2",label="name"),
            hjust=0,vjust=0.5) +
        scale_x_continuous(limits=c(0,3), breaks=c(1,2),
            minor_breaks=NULL, labels=c(label1,label2)) +
        scale_y_continuous(breaks=seq_len(n), minor_breaks=NULL,
            trans="reverse") +
        labs(x="",y="") +
        theme_bw()

    p
}

#' Confident log2 fold changes based on a DESeq2 analysis
#'
#' For all possible absolute log2 fold changes, which genes have at least this
#' fold change at a specified False Discovery Rate? This is built by repeatedly
#' calling DESeq2::results with the "greaterAbs" alternative hypothesis.
#'
#' Results are presented in a table such that for any given LFC, if the reader
#' chooses the genes with abs(confect) less than this they are assured that this
#' set of genes has at least this LFC (with the specified FDR). The confect
#' column may also be viewed as a confidence bound on the LFC of each gene, with
#' a dynamic correction for multiple testing.
#'
#' @param object Object produced by the \code{DESeq2::DESeq} function.
#'
#' @param ... Further arguments to \code{DESeq2::results}. At a minimum you
#'   should specify either \code{contrast=} or \code{name=}.
#'
#' @param fdr False Discovery Rate to control for.
#'
#' @param step Granularity of log2 fold changes to test.
#'
#' @return
#'
#' See \code{\link{nest_confects}} for details of how to interpret the result.
#'
#' The \code{filtered} column in the result indicates whether DESeq2 filtered
#' the gene. Such genes do not count toward the total number of genes when
#' controlling FDR. If your intention is to obtain a ranking of all genes, you
#' should disable this with \code{deseq2_confects(..., cooksCutoff=Inf,
#' independentFiltering=FALSE)}.
#'
#' @examples
#'
#' # Generate some random data
#' n <- 20
#' folds <- seq(-8,8,length.out=n)
#' row_means <- runif(n, min=0, max=5)
#' lib_scale <- c(1,2,3,4)
#' means <- 2^(outer(folds, c(-0.5,-0.5,0.5,0.5))) *
#'     row_means * rep(lib_scale,each=n)
#' counts <- rnbinom(length(means), mu=means, size=1/0.1)
#' dim(counts) <- dim(means)
#'
#' group <- factor(c("A","A","B","B"))
#'
#' # Apply DESeq2
#' library(DESeq2)
#' dds <- DESeqDataSetFromMatrix(
#'     countData = counts,
#'     colData = data.frame(group=group),
#'     design = ~group)
#'
#' dds <- DESeq(dds)
#'
#' # Find top confident effect sizes
#' deseq2_confects(dds, name="group_B_vs_A", step=0.1)
#'
#' @export
deseq2_confects <- function(object, ..., fdr=0.05, step=0.01) {
    assert_that(requireNamespace("DESeq2", quietly=TRUE), msg=
        "deseq2_confects requires installing the Bioconductor package 'DESeq2'")

    n <- nrow(object)

    # Calculate effect column, determine which genes are filtered by DESeq2
    std_result <- DESeq2::results(object, ...)

   # DESeq2 may filter some genes before emitting pvalues
    filtered <- is.na(std_result$padj)
    subset_out <- which(!filtered)
    subset_n <- length(subset_out)

    pfunc <- function(subset_i, mag) {
        i <- subset_out[subset_i]

        result <- DESeq2::results(
            object, ..., altHypothesis="greaterAbs", lfcThreshold=mag)
        result_p <- result$pvalue[i]

        # This should never happen:
        assert_that(!any(is.na(result_p)),
            msg="unexpected extra filtering by DESeq2")

        result_p
    }

    confects <- nest_confects(subset_n, pfunc, fdr=fdr, step=step)
    confects$table$index <- subset_out[ confects$table$index ]

    if (subset_n < n) {
        extra <- data.frame(
            rank=(subset_n+1):n,
            index=which(filtered),
            confect=rep(NA,n-subset_n))
        confects$table <- rbind(confects$table, extra)
    }

    confects$table$effect <- std_result$log2FoldChange[confects$table$index]
    confects$table$confect <- sign(confects$table$effect)*confects$table$confect

    confects$table$baseMean <- std_result$baseMean[confects$table$index]
    confects$table$name <- rownames(object)[confects$table$index]
    confects$table$filtered <- filtered[confects$table$index]

    confects
}

#'Confident effect sizes from from normal or t distributions
#'
#'A general purpose confident effect size function for where a normal or t
#'distribution of errors can be assumed. Calculates confident effect sizes based
#'on an estimated effect and standard deviation (normal distribution), or mean
#'and scale (t distribution).
#'
#'@param effect A vector of estimated effects.
#'
#'@param se A single number or vector of standard errors (or if t distribution,
#'  scales).
#'
#'@param df A single number or vector of degrees of freedom, for t-distribution.
#'  Inf for normal distribution.
#'
#'@param signed If TRUE effects are signed, use TREAT test. If FALSE effects are
#'  all positive, use one sided t-test.
#'
#'@param fdr False Discovery Rate to control for.
#'
#'@param step Granularity of effect sizes to test.
#'
#'@param full Include some further statistics used to calculate confects in the
#'  output, and also include FDR-adjusted p-values that effect size is non-zero
#'  (note that this is against the spirit of the topconfects approach).
#'
#'@return
#'
#'See \code{\link{nest_confects}} for details of how to interpret the result.
#'
#' @examples
#'
#' # Find largest positive or negative z-scores in a collection,
#' # and place confidence bounds on them that maintain FDR 0.05.
#' z <- c(1,-2,3,-4,5)
#' normal_confects(z, se=1, fdr=0.05, full=TRUE)
#'
#'@export
normal_confects <- function(
        effect, se, df=Inf, signed=TRUE,
        fdr=0.05, step=0.001, full=FALSE) {
    n <- max(length(effect), length(se), length(df))
    effect <- broadcast(effect, n)
    se <- broadcast(se, n)
    df <- broadcast(df, n)

    if (signed) {
        abs_effect <- abs(effect)

        pfunc <- function(indices, mag) {
            abs_effect_i <- abs_effect[indices]
            se_i <- se[indices]
            df_i <- df[indices]

            tstat_right <- (abs_effect_i-mag)/se_i
            tstat_left <- (abs_effect_i+mag)/se_i

            pt(tstat_right, df=df_i, lower.tail=FALSE) +
                pt(tstat_left,df=df_i, lower.tail=FALSE)
        }
    } else {
        assert_that(all(effect >= 0.0))

        pfunc <- function(indices, mag) {
            tstat <- (effect[indices]-mag)/se[indices]
            pt(tstat, df=df[indices], lower.tail=FALSE)
        }
    }

    confects <- nest_confects(n, pfunc, fdr=fdr, step=step, full=full)
    ranked_effect <- effect[confects$table$index]
    confects$table$confect <- sign(ranked_effect) * confects$table$confect
    confects$table$effect <- ranked_effect

    if (full) {
        fdr_zero <- confects$table$fdr_zero
        confects$table$fdr_zero <- NULL
        confects$table$se <- se[confects$table$index]
        confects$table$df <- df[confects$table$index]
        confects$table$fdr_zero <- fdr_zero
    }

    if (!signed)
        confects$limits <- c(0,NA)

    confects
}

context("Test missing values handled correctly")

library(topconfects)


effect <- c(1, NA, 10, 3, -20, NA, 7, NA)
se     <- c(1, 0,  2,  2,  1,  0, NA, 1)
good   <- !is.na(effect) & !is.na(se)


res1 <- normal_confects(effect, se, full=TRUE)
sub <- res1$table[ good[res1$table$index], ]

res2 <- normal_confects(effect[good], se[good], full=TRUE)

test_that("results_match", {
    expect_equal(sub$confect, res2$table$confect)
    expect_equal(sub$effect, res2$table$effect)
    expect_equal(sub$se, res2$table$se)
})

---
title: "An overview of topconfects"
author: "Paul Harrison"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{An overview of topconfects}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
params:
    cache: !r FALSE
---

TOP results by CONfident efFECT Size. Topconfects is an R package intended for
RNA-seq or microarray Differntial Expression analysis and similar, where we are
interested in placing confidence bounds on many effect sizes---one per
gene---from few samples, and ranking genes by these confident effect sizes.

Topconfects builds on
[TREAT](http://bioinformatics.oxfordjournals.org/content/25/6/765.long) p-values
offered by the limma and edgeR packages, or the "greaterAbs" test p-values
offered by DESeq2. It tries a range of fold changes, and uses this to rank genes
by effect size while maintaining a given FDR. This also produces confidence
bounds on the fold changes, with adjustment for multiple testing.

* **A principled way to avoid using p-values as a proxy for effect size.** The
difference between a p-value of 1e-6 and 1e-9 has no practical meaning in terms
of significance, however tiny p-values are often used as a proxy for effect
size. This is a misuse, as they might simply reflect greater quality of evidence
(for example RNA-seq average read count or microarray average spot intensity).
It is better to reject a broader set of hypotheses, while maintaining a sensible
significance level.

* **No need to guess the best fold change cutoff.** TREAT requires a fold change
cutoff to be specified. Topconfects instead asks you specify a False Discovery
Rate appropriate to your purpose. You can then read down the resulting ranked
list of genes as far as you wish. The "confect" value given in the last row that
you use is the fold change cutoff required for TREAT to produce that set of
genes at the given FDR.

The method is described in:

[Harrison PF, Pattison AD, Powell DR, Beilharz TH. 2018. Topconfects: a package
for confident effect sizes in differential expression analysis provides improved
usability ranking genes of interest. bioRxiv.
doi:10.1101/343145](https://www.biorxiv.org/content/early/2018/06/11/343145)

## If you want to find top confident differentially expressed genes

Use `limma_confects`, `edger_confects`, or `deseq2_confects` as an alternative
final step in your limma, edgeR, or DESeq2 analysis. The limma method is
currently much faster than other methods.

For examples, see the vignette "Confident fold change".

## If you have a collection of effect sizes with standard errors

If you have a collection of effect sizes of some sort, with associated standard
errors, and possibly associated degrees of freedom, use `normal_confects`.
Errors are assumed to be normally distributed, or t-distributed if degrees of
freedom are given.

This is a re-implementation of limma's TREAT method, which is then supplied to
`nest_confects` (described next). (Alternatively, if the effect sizes are all
positive, there is an option to use a one-sided t-test as the underlying
hypothesis test.)

## If you can calculate p-values for a collection of interval hypotheses

The core algorithm of `topconfects` is implemented in the function
`nest_confects`. You may supply any function that can calculate p-values for the
null hypothesis that an effect size is no more than a specified amount. Testing
is performed for n items, and the function should be able to perform this
calculation for a subset of these n items and a given amount.

## Visualizing results

Use `confects_plot` to plot confident effect sizes of top genes. The estimated
effect size (eg log fold change) is shown as a dot, and the confidence bound is
shown as a line.

Use `confects_plot_me2` to gain a global overview. Similar to an MD or MA plot,
the x axis is average expression and the y axis is estimated effect size. The 
confident effect size is shown using colors. Effect sizes confidently "> 0"
correspond to traditional differential expression testing, and effect sizes
confidently greater than larger values correspond to the TREAT test at that
threshold.

There is also an older `confects_plot_me`, which I no longer recommend as it is
hard to explain and easily misleading. The y axis is effect size. Estimated effect
sizes are shown in grey and confident effect sizes in red or blue (ie a gene with a
non-NA confident effect size is shown with both a grey and a colored dot).

Use `rank_rank_plot` to compare two rankings.

For examples, see the vignette "Confident fold change".

---
title: "Confident fold change"
author: "Paul Harrison"
date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document
vignette: >
    %\VignetteIndexEntry{Confident fold change}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
params:
    cache: !r FALSE
---

This document shows typical Topconfects usage with limma, edgeR, or DESeq2.

The first step is to load a dataset. Here, we're looking at RNA-seq data that
investigates the response of *Arabodopsis thaliana* to a bacterial pathogen.
Besides the experimental and control conditions, there is also a batch effect.
This dataset is also examined in section 4.2 of the `edgeR` user manual, and
I've followed the initial filtering steps in the `edgeR` manual.

```{r setup, include=FALSE}
knitr::opts_chunk$set(fig.width=7, fig.height=6)
```


```{r load, message=FALSE, warning=FALSE}
library(topconfects)

library(NBPSeq)
library(edgeR)
library(limma)

library(dplyr)
library(ggplot2)

data(arab)

# Retrieve symbol for each gene
info <- 
    AnnotationDbi::select(
        org.At.tair.db::org.At.tair.db, 
        rownames(arab), "SYMBOL") %>%
    group_by(TAIR) %>% 
    summarize(
        SYMBOL=paste(unique(na.omit(SYMBOL)),collapse="/"))
arab_info <- 
    info[match(rownames(arab),info$TAIR),] %>% 
    select(-TAIR) %>%
    as.data.frame
rownames(arab_info) <- rownames(arab)

# Extract experimental design from sample names
Treat <- factor(substring(colnames(arab),1,4)) %>% relevel(ref="mock")
Time <- factor(substring(colnames(arab),5,5))

y <- DGEList(arab, genes=as.data.frame(arab_info))

# Keep genes with at least 3 samples having an RPM of more than 2
keep <- rowSums(cpm(y)>2) >= 3
y <- y[keep,,keep.lib.sizes=FALSE]
y <- calcNormFactors(y)
```


# limma analysis

## Standard limma analysis steps

We use the voom-limma method.

```{r limma}
design <- model.matrix(~Time+Treat)
design[,]

fit <-
    voom(y, design) %>%
    lmFit(design)
```

## Apply topconfects

Find largest fold changes that we can be confident of at FDR 0.05.

```{r limma_confects}
confects <- limma_confects(fit, coef="Treathrcc", fdr=0.05)

confects
```

Note: If not using `voom` or a similar precision weighting method, it may be important to use `limma_confects(..., trend=TRUE)` to account for a mean-variance trend, similar to how you would call `limma::treat(..., trend=TRUE)`. You can check the need for this with `limma::plotSA(fit)`.


## Looking at the result

Here the usual `logFC` values estimated by `limma` are shown as dots, with lines
to the `confect` value.

```{r fig.height=7}
confects_plot(confects)
```

`confects_plot_me2` produces a Mean-Difference Plot (as might be produced by `limma::plotMD`),
and colors points based on thresholds for the `confect` values.
Effect sizes confidently "> 0" correspond to traditional differential expression testing, 
and effect sizes confidently greater than larger values correspond to the TREAT test at that
threshold.
I have specified the `breaks` here explicitly, 
to aid comparison with other methods in similar plots below.

```{r}
confects_plot_me2(confects, breaks=c(0,1,2))
```

You can see that while large estimated effect sizes are produced at low expression levels, 
this is mostly due to noise in the estimates. They are not *confidently* large expression levels.

There is also a `confects_plot_me`, which I no longer recommend. 
This overlays the confects (red/blue) on a Mean-Difference Plot (grey). 
This proved to be confusing, as significant gene are represented by two different points.

```{r}
confects_plot_me(confects) + labs(title="This plot is not recommended")
```

Let's compare this to the ranking we obtain from `topTable`.

```{r, fig.height=7}
fit_eb <- eBayes(fit)
top <- topTable(fit_eb, coef="Treathrcc", n=Inf)

rank_rank_plot(confects$table$name, rownames(top), "limma_confects", "topTable")
```

You can see that the top 19 genes from topTable are all within the top 40 for
topconfects ranking, but topconfects has also highly ranked some other genes.
These have a large effect size, and sufficient if not overwhelming evidence of
this.

An MD-plot highlighting the positions of the top 40 genes in both rankings also
illustrates the differences between these two ways of ranking genes.

```{r}
plotMD(fit, legend="bottomleft", status=paste0(
    ifelse(rownames(fit) %in% rownames(top)[1:40], "topTable ",""),
    ifelse(rownames(fit) %in% confects$table$name[1:40], "confects ","")))
```


# edgeR analysis

An analysis in edgeR produces similar results. Note that only quasi-likelihood
testing from edgeR is supported.

## Standard edgeR analysis

```{r edger}
y <- estimateDisp(y, design, robust=TRUE)
efit <- glmQLFit(y, design, robust=TRUE)
```

## Apply topconfects

A step of 0.05 is used here merely so that the vignette will build quickly. 
`edger_confects` calls `edgeR::glmTreat` repeatedly, which is necessarily slow.
In practice a smaller value such as 0.01 should be used.

```{r edger_confects}
econfects <- edger_confects(efit, coef="Treathrcc", fdr=0.05, step=0.05)

econfects
```


## Looking at the result

```{r fig.height=7}
confects_plot(econfects)
confects_plot_me2(econfects, breaks=c(0,1,2))
```

```{r}
etop <-
    glmQLFTest(efit, coef="Treathrcc") %>%
    topTags(n=Inf)

plotMD(efit, legend="bottomleft", status=paste0(
    ifelse(rownames(efit) %in% econfects$table$name[1:40], "confects ", ""),
    ifelse(rownames(efit) %in% rownames(etop)[1:40], "topTags ","")))
```


# DESeq2 analysis

DESeq2 does its own filtering of lowly expressed genes, so we start from the
original count matrix. The initial steps are as for a normal DESeq2 analysis.

```{r message=F, warning=F}
library(DESeq2)

dds <- DESeqDataSetFromMatrix(
    countData = arab,
    colData = data.frame(Time, Treat),
    rowData = arab_info,
    design = ~Time+Treat)

dds <- DESeq(dds)
```

## Apply topconfects

The contrast or coefficient to test is specified as in the `DESeq2::results`
function. The step of 0.05 is merely so that this vignette will build quickly, 
in practice a smaller value such as 0.01 should be used. `deseq2_confects` 
calls `results` repeatedly, and in fairness `results` 
has not been optimized for this.

```{r}
dconfects <- deseq2_confects(dds, name="Treat_hrcc_vs_mock", step=0.05)
```

DESeq2 offers shrunken estimates of LFC. This is another sensible way of ranking
genes. Let's compare them to the confect values.

```{r}
shrunk <- lfcShrink(dds, coef="Treat_hrcc_vs_mock", type="ashr")
dconfects$table$shrunk <- shrunk$log2FoldChange[dconfects$table$index]

dconfects
```

DESeq2 filters some genes, these are placed last in the table. If your intention
is to obtain a ranking of all genes, you should disable this with
`deseq2_confects(..., cooksCutoff=Inf, independentFiltering=FALSE)`.

```{r}
table(dconfects$table$filtered)
tail(dconfects$table)
```

## Looking at the result

Shrunk LFC estimates are shown in red.

```{r fig.height=7}
confects_plot(dconfects) + 
    geom_point(aes(x=shrunk, size=baseMean, color="lfcShrink"), alpha=0.75)
```

`lfcShrink` aims for a best estimate of the LFC, whereas confect is a
conservative estimate. `lfcShrink` can produce non-zero values for genes which
can't be said to significantly differ from zero -- it doesn't do double duty as
an indication of significance -- whereas the confect value will be `NA` in this
case. The plot below compares these two quantities. Only un-filtered genes are
shown (see above).

```{r}
filter(dconfects$table, !filtered) %>%
ggplot(aes(
        x=ifelse(is.na(confect),0,confect), y=shrunk, color=!is.na(confect))) +
    geom_point() + geom_abline() + coord_fixed() + theme_bw() +
    labs(color="Significantly\nnon-zero at\nFDR 0.05", 
        x="confect", y="lfcShrink using ashr")
```

# Comparing results

The `rank_rank_plot` function can be used to pick differences between the different methods. It seems edgeR and DESeq2 tend to promote some genes, relative to limma. edgeR and DESeq2 are more similar to each other, which is reassuring given that they are based on similar statistical models.

```{r fig.height=7}
rank_rank_plot(confects$table$name, econfects$table$name, 
    "limma confects", "edgeR confects")
rank_rank_plot(confects$table$name, dconfects$table$name, 
    "limma confects", "DESeq2 confects")
rank_rank_plot(econfects$table$name, dconfects$table$name, 
    "edgeR confects", "DESeq2 confects")
```

---

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normal.R
\name{normal_confects}
\alias{normal_confects}
\title{Confident effect sizes from from normal or t distributions}
\usage{
normal_confects(
  effect,
  se,
  df = Inf,
  signed = TRUE,
  fdr = 0.05,
  step = 0.001,
  full = FALSE
)
}
\arguments{
\item{effect}{A vector of estimated effects.}

\item{se}{A single number or vector of standard errors (or if t distribution,
scales).}

\item{df}{A single number or vector of degrees of freedom, for t-distribution.
Inf for normal distribution.}

\item{signed}{If TRUE effects are signed, use TREAT test. If FALSE effects are
all positive, use one sided t-test.}

\item{fdr}{False Discovery Rate to control for.}

\item{step}{Granularity of effect sizes to test.}

\item{full}{Include some further statistics used to calculate confects in the
output, and also include FDR-adjusted p-values that effect size is non-zero
(note that this is against the spirit of the topconfects approach).}
}
\value{
See \code{\link{nest_confects}} for details of how to interpret the result.
}
\description{
A general purpose confident effect size function for where a normal or t
distribution of errors can be assumed. Calculates confident effect sizes based
on an estimated effect and standard deviation (normal distribution), or mean
and scale (t distribution).
}
\examples{

# Find largest positive or negative z-scores in a collection,
# and place confidence bounds on them that maintain FDR 0.05.
z <- c(1,-2,3,-4,5)
normal_confects(z, se=1, fdr=0.05, full=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nest_confects.R
\name{nest_confects}
\alias{nest_confects}
\title{General purpose function to find confident effect sizes, controlling FDR}
\usage{
nest_confects(n, pfunc, fdr = 0.05, step = 0.001, full = FALSE)
}
\arguments{
\item{n}{Number of items being tested.}

\item{pfunc}{A \code{function(indices, effect_size)} to calculate p-values.
Indices is a subset of 1:n giving the p-values to be computed. Should
return a numeric vector of length \code{length(indices)}.
May return NA values. 
If an NA value is returned, NA must be returned for all effect sizes. 
NA values are not counted toward the total number of tests performed.}

\item{fdr}{False Discovery Rate to control for.}

\item{step}{Granularity of effect sizes to test.}

\item{full}{If TRUE, also include FDR-adjusted p-value that effect size is
non-zero. Note that this is against the spirit of the topconfects approach.}
}
\value{
A "Topconfects" object, containing a table of results and various associated
information.

The most important part of this object is the $table element, a data frame
with the following columns:

\itemize{ \item \code{rank} - Ranking by \code{confect} and for equal
\code{confect} by p-value at that effect size. \item \code{index} - Number of
the test, between 1 and n. \item \code{confect} - CONfident efFECT size. }

The usage is as follows: To find a set of tests which have effect size
greater than x with the specified FDR, take the rows with \code{abs(confect)
>= x}. Once the set is selected, the confect values provide confidence bounds
on the effect size with False Coverage-statement Rate (FCR) at the same level
as the FDR.

One may essentially take the top however many rows of the data frame and
these will be the best set of results of that size to dependably have an
effect size that is as large as possible. However if some genes have the same
\code{abs(confect)}, all or none should be selected.

Some rows in the output may be given the same \code{confect}, even if
\code{step} is made small. This is an expected behaviour of the algorithm.
(This is similar to FDR adjustment of p-values sometimes resulting in a run
of the same adjusted p-value, even if all the input p-values are distinct.)

Some wrappers around this function may add a sign to the \code{confect}
column, if it makes sense to do so. They will also generally add an
\code{effect} column, containing an estimate of the effect size that aims to
be unbiassed rather than a conservative lower bound.
}
\description{
Find sets of discoveries for a range of effect sizes, controlling the False
Discovery Rate (FDR) for each set.
}
\details{
This is a general purpose function, which can be applied to any method of
calculting p-values (supplied as a function argument) for the null hypothesis
that the effect size is smaller than a given amount.
}
\examples{

# Find largest positive z-scores in a collection,
# and place confidence bounds on them that maintain FDR 0.05.
z <- c(1,2,3,4,5)
pfunc <- function(i, effect_size) {
    pnorm(z[i], mean=effect_size, lower.tail=FALSE)
}
nest_confects(length(z), pfunc, fdr=0.05)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edger.R
\name{edger_confects}
\alias{edger_confects}
\title{Confident log2 fold change based on the edgeR Quasi-Likelihood method}
\usage{
edger_confects(
  fit,
  coef = NULL,
  contrast = NULL,
  fdr = 0.05,
  step = 0.01,
  null = c("worst.case", "interval")
)
}
\arguments{
\item{fit}{An edgeR DGEGLM object produced using \code{glmQLFit}.}

\item{coef}{Coefficient to test, as per \code{glmTreat}. Use either coef or
contrast or effect.}

\item{contrast}{Contrast to test, as per \code{glmTreat}. Use either coef or
contrast or effect.}

\item{fdr}{False Discovery Rate to control for.}

\item{step}{Granularity of log2 fold changes to test.}

\item{null}{"null" parameter passed through to edger::glmTreat (if coef or
contrast given). Choices are "worst.case" or "interval". Note that the
default here is "worst.case", to be consistent with other functions in
topconfects. This differs from the default for glmTreat.}
}
\value{
See \code{\link{nest_confects}} for details of how to interpret the result.
}
\description{
For all possible absolute log2 fold changes (LFC), which genes have at least
this fold change at a specified False Discovery Rate?
}
\details{
Results are presented in a table such that for any given LFC, if the reader
chooses the genes with abs(confect) less than this they are assured that this
set of genes has at least this LFC (with the specified FDR). The confect
column may also be viewed as a confidence bound on the LFC of each gene, with
a dynamic correction for multiple testing.
}
\examples{

# Generate some random data
n <- 100
folds <- seq(-4,4,length.out=n)
row_means <- runif(n, min=0, max=5)
lib_scale <- c(1,2,3,4)
means <- 2^(outer(folds, c(-0.5,-0.5,0.5,0.5))) *
    row_means * rep(lib_scale,each=n)
counts <- rnbinom(length(means), mu=means, size=1/0.1)
dim(counts) <- dim(means)

design <- cbind(c(1,1,0,0), c(0,0,1,1))

# Fit data using edgeR quasi-likelihood
library(edgeR)
y <- DGEList(counts)
y <- calcNormFactors(y)
y <- estimateDisp(y, design)
fit <- glmQLFit(y, design)

# Find top confident effect sizes
edger_confects(fit, contrast=c(-1,1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deseq2.R
\name{deseq2_confects}
\alias{deseq2_confects}
\title{Confident log2 fold changes based on a DESeq2 analysis}
\usage{
deseq2_confects(object, ..., fdr = 0.05, step = 0.01)
}
\arguments{
\item{object}{Object produced by the \code{DESeq2::DESeq} function.}

\item{...}{Further arguments to \code{DESeq2::results}. At a minimum you
should specify either \code{contrast=} or \code{name=}.}

\item{fdr}{False Discovery Rate to control for.}

\item{step}{Granularity of log2 fold changes to test.}
}
\value{
See \code{\link{nest_confects}} for details of how to interpret the result.

The \code{filtered} column in the result indicates whether DESeq2 filtered
the gene. Such genes do not count toward the total number of genes when
controlling FDR. If your intention is to obtain a ranking of all genes, you
should disable this with \code{deseq2_confects(..., cooksCutoff=Inf,
independentFiltering=FALSE)}.
}
\description{
For all possible absolute log2 fold changes, which genes have at least this
fold change at a specified False Discovery Rate? This is built by repeatedly
calling DESeq2::results with the "greaterAbs" alternative hypothesis.
}
\details{
Results are presented in a table such that for any given LFC, if the reader
chooses the genes with abs(confect) less than this they are assured that this
set of genes has at least this LFC (with the specified FDR). The confect
column may also be viewed as a confidence bound on the LFC of each gene, with
a dynamic correction for multiple testing.
}
\examples{

# Generate some random data
n <- 20
folds <- seq(-8,8,length.out=n)
row_means <- runif(n, min=0, max=5)
lib_scale <- c(1,2,3,4)
means <- 2^(outer(folds, c(-0.5,-0.5,0.5,0.5))) *
    row_means * rep(lib_scale,each=n)
counts <- rnbinom(length(means), mu=means, size=1/0.1)
dim(counts) <- dim(means)

group <- factor(c("A","A","B","B"))

# Apply DESeq2
library(DESeq2)
dds <- DESeqDataSetFromMatrix(
    countData = counts,
    colData = data.frame(group=group),
    design = ~group)

dds <- DESeq(dds)

# Find top confident effect sizes
deseq2_confects(dds, name="group_B_vs_A", step=0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topconfects.R
\name{confects_plot}
\alias{confects_plot}
\title{Top confident effect sizes plot}
\usage{
confects_plot(confects, n = 50, limits = NULL)
}
\arguments{
\item{confects}{A "Topconfects" class object, as returned from
limma_confects, edger_confects, etc.}

\item{n}{Number if items to show.}

\item{limits}{c(lower, upper) limits on x-axis.}
}
\value{
A ggplot2 object. Working non-interactively, you must print() this for it to
be displayed.
}
\description{
Create a ggplot2 object showing the confect, effect, and average expression
level of top features in a Topconfects object.
}
\details{
For each gene, the estimated effect is shown as a dot. The confidence bound
is shown as a line to positive or negative infinity, showing the set of
non-rejected effect sizes for the feature.
}
\examples{

# Generate some random effect sizes with random accuracies
n <- 100
effect <- rnorm(n, sd=2)
se <- rchisq(n, df=3)^-0.5

# Find top confident effect sizes
confects <- normal_confects(effect, se)

# Plot top confident effect sizes
confects_plot(confects, n=30)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topconfects.R
\name{confects_plot_me2}
\alias{confects_plot_me2}
\title{Mean-expression vs effect size plot (version 2)}
\usage{
confects_plot_me2(confects, breaks = NULL, signed_colors = TRUE)
}
\arguments{
\item{confects}{A "Topconfects" class object, as returned from
\code{limma_confects}, \code{edger_confects}, or \code{deseq2_confects}.}

\item{breaks}{A vector of non-negative confect thresholds to color points with.
Chooses a sensible set of breaks if none are given.}

\item{signed_colors}{Should positive and negative confects be colored 
distinctly, red and blue.}
}
\value{
A ggplot2 object. Working non-interactively, you must print() this for it to
be displayed.
}
\description{
Like plotMD in limma, plots effect size against mean expression level.
Confect values are indicated using color.
This may be useful for assessing whether effects are
only being detected in highly expressed genes.
}
\examples{

library(NBPSeq)
library(edgeR)
library(limma)

data(arab)

# Extract experimental design from sample names
treat <- factor(substring(colnames(arab),1,4), levels=c("mock","hrcc"))
time <- factor(substring(colnames(arab),5,5))

# Keep genes with at least 3 samples having an RPM of more than 2
y <- DGEList(arab)
keep <- rowSums(cpm(y)>2) >= 3
y <- y[keep,,keep.lib.sizes=FALSE]
y <- calcNormFactors(y)

# Find top confident fold changes by topconfects-limma-voom method
design <- model.matrix(~time+treat)
voomed <- voom(y, design)
fit <- lmFit(voomed, design)
confects <- limma_confects(fit, "treathrcc")

# Plot confident effect size against mean expression
# (estimated effect size also shown as grey dots)
confects_plot_me2(confects)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topconfects.R
\name{rank_rank_plot}
\alias{rank_rank_plot}
\title{A plot to compare two rankings}
\usage{
rank_rank_plot(
  vec1,
  vec2,
  label1 = "First ranking",
  label2 = "Second ranking",
  n = 40
)
}
\arguments{
\item{vec1}{A vector of names.}

\item{vec2}{Another vector of names.}

\item{label1}{A label to go along with vec1.}

\item{label2}{A label to go along with vec2.}

\item{n}{Show at most the first n names in vec1 and vec2.}
}
\value{
A ggplot2 object. Working non-interactively, you must print() this for it to
be displayed.
}
\description{
This is useful, for example, when comparing different methods of ranking
potentially interesting differentially expressed genes.
}
\examples{

a <- sample(letters)
b <- sample(letters)
rank_rank_plot(a,b, n=20)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topconfects.R
\name{confects_plot_me}
\alias{confects_plot_me}
\title{Mean-expression vs effect size plot (deprecated)}
\usage{
confects_plot_me(confects)
}
\arguments{
\item{confects}{A "Topconfects" class object, as returned from
\code{limma_confects}, \code{edger_confects}, or \code{deseq2_confects}.}
}
\value{
The two types of points in this plot make it quite confusing to explain.
\code{plot_confects_me2} is recommended instead.

A ggplot2 object. Working non-interactively, you must print() this for it to
be displayed.
}
\description{
Note: I now recommend using \code{plot_confects_me2} instead of this plot. 
Like plotMD in limma, plots effect size against mean expression level.
However shows "confect" on the y axis rather than "effect" ("effect" is shown
underneath in grey). This may be useful for assessing whether effects are
only being detected in highly expressed genes.
}
\examples{

library(NBPSeq)
library(edgeR)
library(limma)

data(arab)

# Extract experimental design from sample names
treat <- factor(substring(colnames(arab),1,4), levels=c("mock","hrcc"))
time <- factor(substring(colnames(arab),5,5))

# Keep genes with at least 3 samples having an RPM of more than 2
y <- DGEList(arab)
keep <- rowSums(cpm(y)>2) >= 3
y <- y[keep,,keep.lib.sizes=FALSE]
y <- calcNormFactors(y)

# Find top confident fold changes by topconfects-limma-voom method
design <- model.matrix(~time+treat)
voomed <- voom(y, design)
fit <- lmFit(voomed, design)
confects <- limma_confects(fit, "treathrcc")

# Plot confident effect size against mean expression
# (estimated effect size also shown as grey dots)
confects_plot_me(confects)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limma.R
\name{limma_confects}
\alias{limma_confects}
\title{Confident log2 fold changes based on a limma fit object}
\usage{
limma_confects(
  fit,
  coef = NULL,
  fdr = 0.05,
  step = 0.001,
  trend = FALSE,
  full = FALSE
)
}
\arguments{
\item{fit}{A limma MArrayLM object.}

\item{coef}{Number or name of coefficient or contrast to test.}

\item{fdr}{False Discovery Rate to control for.}

\item{step}{Granularity of log2 fold changes to test.}

\item{trend}{Should \code{eBayes(trend=TRUE)} be used?}

\item{full}{Include some further statistics used to calculate confects in the
output, and also include FDR-adjusted p-values that effect size is non-zero
(note that this is against the spirit of the topconfects approach).}
}
\value{
See \code{\link{nest_confects}} for details of how to interpret the result.
}
\description{
For all possible absolute log2 fold changes (LFC), which genes have at least
this fold change at a specified False Discovery Rate (FDR)?
}
\details{
Results are presented in a table such that for any given LFC, if the reader
chooses the genes with \code{abs(confect)} less than this they are assured
that this set of genes has at least this LFC (with the specified FDR). Once
this set of genes is selected, the confect values provide confidence bounds
with False Coverage-statement Rate at the same level as the FDR.

\code{fit} should be produced using \code{lmFit}. It is not necessary to use
\code{eBayes}, this function calls \code{eBayes} itself.

To test contrasts, this function can also be used with the result of
\code{contrasts.fit}, but limma's handling of weights may be approximate (for
example if \code{voom} has been used). For exact results for a contrast, use
\code{contrastToCoef} to adjust the design matrix given to \code{lmFit}.
}
\examples{

#Prepare a data set
library(NBPSeq)
library(edgeR)
library(limma)
data(arab)
dgelist <- DGEList(arab)
dgelist <- calcNormFactors(dgelist)
cpms <- cpm(dgelist, log=TRUE)
# Retain genes with more than a geometric mean of 2 RPM
# (about 5 reads per sample)
cpms <- cpms[rowMeans(cpms) >= 1,]

# Fit linear model for each gene
treatment <- c(FALSE,FALSE,FALSE,TRUE,TRUE,TRUE)
batch <- factor(c(1,2,3,1,2,3))
design <- model.matrix(~ treatment + batch)
fit <- lmFit(cpms, design)

# Calculate top confects
# As voom has not been used, it is necessary to use trend=TRUE
limma_confects(fit, "treatmentTRUE", trend=TRUE)

}
